import * as angular from 'angular';

export default class NwfFormElementController {

    element: any;

    static $inject = ['$element', '$window', '$attrs'];

    constructor(private $element: angular.IAugmentedJQuery, private $window, private $attrs) {

        this.element = $element.get(0);

        // Observing 'disabled' attribute on element, and removing invalidState on the element if its disabled.
        $attrs.$observe('disabled', (newVal) => {
            if (newVal) {
                this.removeInvalidState();
            }
        });

        // On input: Now I connect the events oninput provides real time immediate feedback on each key
        var inputHandler = this.updateValidity.bind(this);
        this.element.addEventListener('input', inputHandler);
        // On invalid
        var invalidHandler = this.invalidHandler.bind(this);
        this.element.addEventListener('invalid', invalidHandler);
        // On blur
        var blurHandler = this.blurHandler.bind(this);
        this.element.addEventListener('blur', blurHandler);
        // On focus
        var focusHandler = this.focusHandler.bind(this)
        this.element.addEventListener('focus', focusHandler);
        //on Change
        var changeHandler = this.changeHandler.bind(this);
        this.element.addEventListener('change', changeHandler);
        $element.on('$destroy', () => {
            this.element.removeEventListener('input', inputHandler);
            this.element.removeEventListener('invalid', invalidHandler);
            this.element.removeEventListener('blur', blurHandler);
            this.element.removeEventListener('focus', focusHandler);
            this.element.removeEventListener('change', changeHandler);
        });
    }

    //Create invalid icon.
    setInvalidState() {
        if (!this.element.invalidIcon) {
            this.element.classList.add('is-invalid');
            var i = document.createElement('i');
            i.className = 'is-invalid-icon icon-Error-Full';
            this.element.parentNode.insertBefore(i, this.element.nextSibling);
            this.element.invalidIcon = i;
        }
    };

    //Remove invalid icon.
    removeInvalidState() {
        // Delete the invalid icon node.
        if (this.element.invalidIcon) {
            this.element.classList.remove('is-invalid');
            this.element.invalidIcon.parentNode.removeChild(this.element.invalidIcon);
            delete this.element.invalidIcon;
        }
    };

    //Show the tooltip on the DOM
    showToolTip() {
        if (this.element.reportValidity) {
            this.element.reportValidity();
            return;
        }
        //var w = window;
        var w = this.$window;
        var isIE = w.ActiveXObject || "ActiveXObject" in w;
        var isFF = w.navigator.userAgent.toLowerCase().indexOf('firefox') > -1;

        if (this.element.submitButtonClicked && (isIE || isFF)) {
            if (isFF) {
                delete this.element.submitButtonClicked;
            }
            return;
        }

        /* Showing the tip ourselves! */
        if (!this.element.myValidationTooltip) {
            this.element.parentNode.classList.add("tooltip-wrapper");
            var i = document.createElement('div');
            i.className = 'report-validity-tooltip';
            i.innerHTML = this.element.validationMessage;
            // Add the tip to the dom
            this.element.parentNode.appendChild(i);
            this.element.myValidationTooltip = i;
        }
    };

    /**
     * Removes the tooltip.
     * @private
     */
    removeTip() {
        if (this.element.myValidationTooltip) {
            this.element.parentNode.classList.remove("tooltip-wrapper");
            this.element.myValidationTooltip.parentNode.removeChild(this.element.myValidationTooltip);
            delete this.element.myValidationTooltip;
        }
    };

    //updateValidity can be called at will to update the ui affordances
    updateValidity(isFocused) {

        /* First I show my tip, if I am invalid and focused, and I don't show tips on Selects, since they are covered up anyway by the select menu itself. */
        if (!this.element.validity.valid && this.element.hasBeenBlurred && isFocused && this.element.tagName !== 'SELECT') {
            this.showToolTip();
        } else {
            this.removeTip();
        }

        /* If I am not valid so set my state if I am not valid and have been blurred, or not valid and submitClicked (set in invalid handler) */
        if (!this.element.validity.valid && this.element.hasBeenBlurred || !this.element.validity.valid && this.element.submitButtonClicked) {
            this.setInvalidState();
        } else {
            this.removeInvalidState();
        }
    };

    //Blur Handler
    blurHandler() {
        /* We set this flag so that when first go in, it doesn't yell. Don't yell I haven't done anything wrong yet! */
        if (!this.element.hasBeenBlurred) {
            this.element.hasBeenBlurred = true;
        }
        this.updateValidity(false);
    };

    //Focus Handler
    focusHandler() {
        this.updateValidity(true);
    };

    invalidHandler() {
        this.element.submitButtonClicked = true;
        /* We have to call this to apply our states when native val happens */
        this.updateValidity(false);
    };


    changeHandler() {
        if (this.element.tagName == 'SELECT') {
            if (this.element.value) {
                this.removeInvalidState();
                this.element.setCustomValidity('');
            }
            else {
                this.setInvalidState();
            }
        }
    };
}