export default class NwfIconController {

    private isTwoColor: boolean;
    private indicatorClass: string;
    private backgroundClass: string;
    private iconClass: string;

    static $inject = ['$attrs', '$timeout'];

    constructor($attrs, $timeout) {
        this.update(this.iconClass);

        $attrs.$observe('iconClass', (newIconClass) => {
            $timeout(() => {
                this.update(newIconClass);
            });
        });
    }

    update(iconClass) {

        /* gets the TwoColorBackgrounClass for this icon */
        function getTwoColorBackgroundClass(iconClass) {
            var stopSignIcon = 'icon-Critical-Error-Stop-Sign',
                circleIcon = 'icon-CircleBackGround',
                triangleIcon = 'icon-WarningTriangle';

            var twoColorIconBackgroundMap = {
                'icon-Critical-Full': stopSignIcon,
                'icon-Help': circleIcon,
                'icon-Error-Full': stopSignIcon,
                'icon-OkCircleWithCheck': circleIcon,
                'icon-Info': circleIcon,
                'icon-Warning': triangleIcon
            };

            if (iconClass in twoColorIconBackgroundMap) {
                return twoColorIconBackgroundMap[iconClass];
            }
            return undefined;
        }

        function getTwoColorIndicatorClass(iconClass) {
            var criticalIcon = 'icon-Critical-X',
                helpIcon = 'icon-QuestionMark',
                checkMarkIcon = 'icon-OkCheckMark',
                exclamationIcon = 'icon-Error-Exclamation',
                infoIcon = 'icon-i-Info';

            var twoColorIconIndicatorMap = {
                'icon-Critical-Full': criticalIcon,
                'icon-Help': helpIcon,
                'icon-Error-Full': exclamationIcon,
                'icon-OkCircleWithCheck': checkMarkIcon,
                'icon-Info': infoIcon,
                'icon-Warning': exclamationIcon
            };

            if (iconClass in twoColorIconIndicatorMap) {
                return twoColorIconIndicatorMap[iconClass];
            }
            return undefined;
        }

        /* Now I find out if it is a two color */
        this.isTwoColor = false;
        var iconCls = this.iconClass;
        var indicatorClass = getTwoColorIndicatorClass(iconCls);
        var backgroundClass = getTwoColorBackgroundClass(iconCls);

        if (indicatorClass && backgroundClass) {
            this.isTwoColor = true;
            this.indicatorClass = indicatorClass;
            this.backgroundClass = backgroundClass;

        } else {
            this.indicatorClass = iconCls;
        }
    }
}
