 
/// <amd-dependency path="text!./nwfLogin-failed-dialog.html" name="loginFailedModalTemplate" />
/// <amd-dependency path="text!./nwfLogin-forgot-password-info.html" name="loginForgotPasswordTemplate" />
/// <amd-dependency path="text!./nwfLogin-reset-password-info.html" name="loginResetPasswordTemplate" />
/// <amd-dependency path="text!./nwfResetPasswordResult-modal.html" name="loginResetPasswordResultTemplate" />
   
   
declare var loginFailedModalTemplate;
declare var loginForgotPasswordTemplate;
declare var loginResetPasswordTemplate;
declare var loginResetPasswordResultTemplate;

interface stateParams {
    user: string;
    authToken: string;
}

export default class NwfLoginController {
    static $inject = ['$scope', 'nwfLoginService', '$uibModal', '$rootScope', '$element', 'authStrategy'];
    constructor($scope, nwfLoginService, $uibModal, $rootScope, $element, authStrategy) {

        var errorCode;
        // Controller used by login failed dialog.
        var loginFailedController = ['$scope', '$uibModalInstance',
            function($scope, $uibModalInstance) {
                $scope.errorCode = errorCode;
                $scope.ok = function() {
                    $uibModalInstance.close();
                };
            }
        ];

        //Controller for forgotPassword modal instance
        var forgotPasswordCtrl = function($scope, forgotPasswordName) {
            $scope.username = forgotPasswordName;
        };

        // Controller used by reset password modal instance on success. The user is redirected to login page.
        var resetPasswordSuccessController = ['$scope', '$uibModalInstance',
            function($scope, $uibModalInstance) {
                $scope.ok = function() {
                    $uibModalInstance.close();
                    window.location.href = '/';
                };
            }
        ];

        // View model used by template.
        //
        $scope.titleElement = null;

        $scope.username = null;
        $scope.password = null;


        //Forgot Password.
        //Scope variable for forget password
        $scope.forgotPassword = false;


        //Reset Password.
        //Scope variable for reset password

        $scope.resetPassword = false;
        $scope.authToken = null;
        $scope.resetPasswordMatch = true;

        // If the server sends a username and authToken, the only use case we have is that user has send a request to reset the password with this URL.
        //Set the boolean to reset password to be true.
            

        if (document.location.hash.indexOf('?') !== -1) {
            var $stateParams = document.location.hash.split('?')[1].split('&');

            var stateParams = <stateParams>{};
            $stateParams.forEach(function(item) {
                var pair = item.split('=');
                stateParams[pair[0]] = pair[1];
            });

            if (stateParams.user !== undefined && stateParams.authToken !== undefined) {
                $scope.username = stateParams.user;
                $scope.resetPassword = true;
                $scope.forgotPassword = false;
                $scope.authToken = stateParams.authToken;
                $rootScope.$broadcast('nwfShowLogin');
            }
        }


        var el = $element.get(0);
        /* This is something I'm not happy with. When I reset, I need to flip the hasBeenBlurred bit so my nwfValidation works as if it has been reset
        it is rare that I have a reset case, and for now I just have this on the login form. */
        var frm = <HTMLFormElement>el.querySelector('#nwf-login-form');
        frm.addEventListener('reset', function() {
            for (var i = 0; i < frm.elements.length; i++) {
                if (frm.elements[i]['hasBeenBlurred']) {
                    delete frm.elements[i]['hasBeenBlurred'];
                }
            }
        });


        /* onLoginForm Submit */
        $scope.submitFn = function() {

            $scope.inFlight = true;
            var credentials = {
                username: $scope.username,
                password: $scope.password
            };

            authStrategy.authenticateFn(credentials).then(
                function() {
                    // Nothing to do here.
                    $scope.inFlight = false;
                },
                function(error) {
                    errorCode = error.status;
                    $scope.inFlight = false;
                    $uibModal.open({
                        template: loginFailedModalTemplate,
                        controller: loginFailedController
                    });
                }
            );
        };


        //Function to toggle the scope variable.
        $scope.forgotPasswordToggleFn = function() {
            $scope.forgotPassword = ($scope.forgotPassword === false) ? true : false;
        };

        //Forgot Password cancel action.
        $scope.forgotPasswordCancelFn = function() {
            $scope.forgotPassword = false;
        };


        //forgot password on submit.
        $scope.forgotPasswordSubmitFn = function() {

            var credentials = {
                username: $scope.username
            };

            nwfLoginService.forgotPasswordAuthenticateFn(credentials).then(
                function() {
                    var forgotPasswordToggle = $uibModal.open({
                        template: loginForgotPasswordTemplate,
                        controller: forgotPasswordCtrl,
                        resolve: {
                            forgotPasswordName: function() {
                                return credentials.username;
                            }
                        }
                    });

                    //Once done do back to the Login form.
                    forgotPasswordToggle.result.then(function() {
                        $scope.forgotPasswordToggleFn();
                    });
                },
                function() {
                }
            );
        };

        //Reset password on cancel.
        $scope.resetPasswordCancelFn = function() {
            window.location.href = document.location.origin;
        };

        //Reset password on submit.
        $scope.resetPasswordSubmitFn = function() {
            var data = {
                user: $scope.username,
                newPassword: $scope.newPassword,
                confirmNewPassword: $scope.confirmNewPassword,
                authToken: $scope.authToken

            };

            //Check if passwords match
            if ($scope.newPassword !== $scope.confirmNewPassword) {
                $scope.resetPasswordMatch = false;
                return;
            }
            else {
                $scope.resetPasswordMatch = true;
            }

            nwfLoginService.resetPasswordFn(data).then(
                function() {
                    $scope.message = 'You have successfully reset your password. You’ll now be taken to the Login Screen.';
                    $scope.success = true;

                    //Show the modal with success message from the server.
                    $uibModal.open({
                        template: loginResetPasswordResultTemplate,
                        scope: $scope,
                        controller: resetPasswordSuccessController
                    });
                },
                function(error) {
                    $scope.message = 'Password reset failed.';
                    if (error != null) {
                        $scope.message = error.message;
                    }
                    $scope.success = false;
                    //Show the  modal with failure message from the server.

                    $uibModal.open({
                        template: loginResetPasswordResultTemplate,
                        scope: $scope,
                        controller: loginFailedController
                    });
                }
            );
        };

    }

};

